/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  int SPdQuantL (double x, const double Xq[], int Nreg)

Purpose:
  Binary search for a bounding interval

Description:
  This function returns the index of the quantizer region corresponding to a
  given input value.  The quantizer is specified by an array of quantizer
  decision levels.  A binary search is used to determine the quantizer region.

  The index value takes on values from 0 to Nreg-1.  If Nreg is equal to one,
  the index is set to zero.  Otherwise, the index is determined as shown in
  the following table.  Note that the number of decision levels is one less
  than the number of regions.
    index            SPfQuantL
     0                    x < Xq[0]
     1           Xq[0] <= x < Xq[1]
     2           Xq[1] <= x < Xq[2]
    ...                  ...
     i         Xq[i-1] <= x < Xq[i]
    ...                  ...
   Nreg-2   Xq[Nreg-3] <= x < Xq[Nreg-2]
   Nreg-1   Xq[Nreg-2] <= x

Parameters:
  <-  int SPfQuantL
      Output index of the quantizer region containing the input value x.
      The index takes on values from 0 to Nreg-1.
   -> double x
      Value to be quantized
   -> const double Xq[]
      Array of Nreg-1 quantizer decision levels which delimit the Nreg
      quantizer regions.  These levels must be in ascending order.
   -> int Nreg
      Number of quantizer regions

Author / revision:
  P. Kabal  Copyright (C) 2003
  $Revision: 1.2 $  $Date: 2003/05/09 02:45:14 $

-------------------------------------------------------------------------*/

#include <libtsp.h>


int
SPdQuantL (double x, const double Xq[], int Nreg)

{
  int iL, iU, i;

/* Binary search for the interval (Xq[iL], Xq[iU]] which brackets x */
  iL = 0;
  iU = Nreg;

  /* Isolate the interval */
  while (iU > iL + 1) {
    i = (iL + iU) / 2;
    if (x < Xq[i-1])
      iU = i;
    else
      iL = i;
  }

/* Return the index */
  return iL;
}
